//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.


#include "parallel/PetscSolverFeti.h"
#include "parallel/StdMpi.h"
#include "parallel/MpiHelper.h"

namespace AMDiS {

  using namespace std;


#ifdef HAVE_PETSC_DEV 
  void PetscSolverFeti::updateDofData(int nComponents)
  {
    FUNCNAME("PetscSolverFeti::updateDofData()");
   
    definePrimals();
  }


  void PetscSolverFeti::definePrimals()
  {
    FUNCNAME("PetscSolverFeti::definePrimals()");

    TEST_EXIT_DBG(meshDistributor->getMesh()->getDim() == 2)
      ("Works for 2D problems only!");

    primals = meshDistributor->getBoundaryDofInfo().geoDofs[VERTEX];
    globalPrimalIndex.clear();
   
    int nRankPrimals = 0;
    for (DofContainerSet::iterator it = primals.begin(); 
	 it != primals.end(); ++it)
      if (meshDistributor->getIsRankDof(**it)) {
	globalPrimalIndex[**it] = nRankPrimals;
	nRankPrimals++;
      }

    int nOverallPrimals = 0, rStartPrimals = 0;
    mpi::getDofNumbering(meshDistributor->getMpiComm(),
			 nRankPrimals, rStartPrimals, nOverallPrimals);

    for (DofMapping::iterator it = globalPrimalIndex.begin();
	 it != globalPrimalIndex.end(); ++it)
      it->second += rStartPrimals;

    MSG_DBG("nRankPrimals = %d   nOverallPrimals = %d\n",
	    nRankPrimals, nOverallPrimals);
  }


  void PetscSolverFeti::fillPetscMatrix(Matrix<DOFMatrix*> *mat, SystemVector *vec)
  {
    FUNCNAME("PetscSolverFeti::fillPetscMatrix()");   
  }


  void PetscSolverFeti::solvePetscMatrix(SystemVector &vec, AdaptInfo *adaptInfo)
  {
    FUNCNAME("PetscSolverFeti::solvePetscMatrix()");
  }
#endif

}
